
/**
 * Module dependencies.
 */

var Emitter = require('emitter')
  , Reader = require('./reader');

/**
 * Expose `file()`.
 */

module.exports = file;

/**
 * Initialize a new `File` wrapping `file`.
 *
 * @param {File} file
 * @return {File}
 * @api public
 */

function file(file) {
  return new File(file);
}

/**
 * Initialize a new `File` wrapper.
 *
 * @param {File} file
 * @api private
 */

function File(file) {
  Emitter.call(this);
  this.file = file;
  for (var key in file) this[key] = file[key];
}

/**
 * Inherits from `Emitter.prototype`.
 */

Emitter(File.prototype);

/**
 * Check if the mime type matches `type`.
 *
 * Examples:
 *
 *    file.is('image/jpeg')
 *    file.is('image/*')
 *
 * @param {String} type
 * @return {Boolean}
 * @api public
 */

File.prototype.is = function(type){
  var real = this.file.type;

  // identical
  if (type == real) return true;

  real = real.split('/');
  type = type.split('/');

  // type/*
  if (type[0] == real[0] && type[1] == '*') return true;

  // */subtype
  if (type[1] == real[1] && type[0] == '*') return true;

  return false;
};

/**
 * Convert to `type` and invoke `fn(err, result)`.
 *
 * @param {String} type
 * @param {Function} fn
 * @return {Reader}
 * @api private
 */

File.prototype.to = function(type, fn){
  if (!window.FileReader) return fn();
  var reader = Reader();
  reader.on('error', fn);
  reader.on('end', function(res){ fn(null, res) });
  reader.read(this.file, type);
  return reader;
};

/**
 * Convert to an `ArrayBuffer`.
 *
 * @param {Function} fn
 * @return {Reader}
 * @api public
 */

File.prototype.toArrayBuffer = function(fn){
  return this.to('ArrayBuffer', fn);
};

/**
 * Convert to text.
 *
 * @param {Function} fn
 * @return {Reader}
 * @api public
 */

File.prototype.toText = function(fn){
  // TODO: encoding
  return this.to('Text', fn);
};

/**
 * Convert to a data uri.
 *
 * @param {Function} fn
 * @return {Reader}
 * @api public
 */

File.prototype.toDataURL = function(fn){
  return this.to('DataURL', fn);
};
